unit Hs3dbtn;

interface

uses
  WinTypes, WinProcs, SysUtils, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, HSgrphic;

type
  TButtonType = (btEllipse, btRoundRect, btRect, btDblEllipse, btDblRRect,
                 btInsetEllipse, btInsetRRect, btChord, btInsetRect);
  TFaceType = (ftBlackness, ftWhiteness, ftTwoColors);
  TGlyphLayout = (glGlyphLeft, glGlyphRight, glGlyphTop, glGlyphBottom);

type
  THS3DButton = class(TGraphicControl)
  private
  FStart, FEnd, FSColor, FLight : TColor;
  FStyle : TBevelStyle;
  FBType : TButtonType;
  FFType : TFaceType;
  FWidth, FHeight : Integer;
  FUp, FDown, FFrame : TBitmap;
  FDwn, FBMP, FMouse,
  FMSelect : Boolean;
  FCaption : string;
  FPicture : TPicture;
  FGLayout : TGlyphLayout;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Paint; override;
    procedure SetStartColor(AColor : TColor);
    procedure SetLight(AColor : TColor);
    procedure SetBType(Atype : TButtonType);
    procedure SetFType(Atype : TFaceType);
    procedure SetMouseSelect(ASelect : Boolean);
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure CMMouseEnter(var msg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var msg: TMessage); message CM_MOUSELEAVE;
    procedure MouseUp(Button : TMouseButton; Shift : TShiftState; X, Y : Integer); override;
    procedure MouseDown(Button : TMouseButton; Shift : TShiftState; X, Y : Integer); override;
    procedure SetBounds(aLeft, aTop, aWidth, aHeight: Integer); override;
    procedure DrawBitmaps;
    procedure SetCaption(ACaption : string);
    procedure SetPicture(APicture : TPicture);
    procedure SetLayout(ALayout : TGlyphLayout);
    procedure DrawGlyphAndCaption(Canvas : TCanvas);
    procedure SetFStyle(AStyle : TBevelStyle);
    procedure DrawSelectFace;
    procedure DrawFace;
    procedure ReadUp(Stream: TStream);
    procedure ReadDown(Stream: TStream);
    procedure WriteUp(Stream: TStream);
    procedure WriteDown(Stream: TStream);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property BevelStyle : TBevelStyle read FStyle write SetFStyle;
    property GlyphLayout : TGlyphLayout read FGLayout write SetLayout;
    property Caption: string read FCaption write SetCaption;
    property Glyph : TPicture read FPicture write SetPicture;
    property ButtonType : TButtonType read FBType write SetBType;
    property FaceType : TFaceType read FFType write SetFType;
    property FaceColor : TColor read FStart write SetStartColor;
    property LightColor : TColor read FLight write SetLight;
    property MouseSelect : Boolean read FMSelect write SetMouseSelect;
    property SelectColor : TColor read FSColor write FSColor;
    property Font;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('HS Controls', [THS3DButton]);
end;

constructor THS3DButton.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  Font.Name := 'MS Sans Serif';
  Font.Size := 9;
  Font.Style := [fsBold];
  FPicture := TPicture.Create;
  if csDesigning in ComponentState then
  FCaption := 'HS3DButton';
  FStart := $00474747;
  FEnd := clWhite;
  FLight := $0080FFFF;
  FSColor := $0000DAF2;
  FMSelect := False;
  FFType := ftWhiteness;
  FBType := btEllipse;
  FHeight := -1;
  FWidth := -1;
  FDwn := False;
  FBMP := False;
  FStyle := bsLowered;
  FUp := TBitMap.Create;
  FDown := TBitMap.Create;
  Enabled := True;
  inherited SetBounds(Left, Top, 40, 40);
end;

destructor THS3DButton.Destroy;
begin
  FPicture.Free;
  FUp.Free;
  FDown.Free;
  inherited Destroy;
end;

procedure THS3DButton.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1;
end;

procedure THS3DButton.ReadUp(Stream: TStream);
begin
  FUp.LoadFromStream(Stream);
end;

procedure THS3DButton.ReadDown(Stream: TStream);
begin
  FDown.LoadFromStream(Stream);
end;

procedure THS3DButton.WriteUp(Stream: TStream);
begin
  FUp.SaveToStream(Stream);
end;

procedure THS3DButton.WriteDown(Stream: TStream);
begin
  FDown.SaveToStream(Stream);
end;

procedure THS3DButton.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineBinaryProperty('ImageUp', ReadUp, WriteUp, True);
  Filer.DefineBinaryProperty('ImageDown', ReadDown, WriteDown, True);
end;

procedure THS3DButton.Paint;
begin
  if csLoading in ComponentState then Exit;
  if FMSelect = True then DrawSelectFace
  else DrawFace;
end;

procedure THS3DButton.SetStartColor(AColor : TColor);
begin
   FStart := AColor;
   DrawBitmaps;
   Invalidate;
end;

procedure THS3DButton.DrawBitmaps;
var
  TargetRect : TRect;
  i, RW, RH, RHS, RWS, PX,
  PY, KX, KY, DX, DY  : Integer;
  clrFrom,clrTo : TColor;
  RGBFromR,RGBFromG,RGBFromB : Byte;
  RGBDiffR,RGBDiffG,RGBDiffB : integer;

   procedure DrawBitmapFrame;
      function GetDarkColor(BShadow : TColor) : TColor;
       var
        RGB_R,RGB_G,RGB_B : Byte;
      begin
        RGB_R := GetRValue (ColorToRGB (BShadow)) - GetRValue (ColorToRGB (BShadow)) div 5;
        RGB_G := GetGValue (ColorToRGB (BShadow)) - GetGValue (ColorToRGB (BShadow)) div 5;
        RGB_B := GetBValue (ColorToRGB (BShadow)) - GetBValue (ColorToRGB (BShadow)) div 5;
        Result := TColor(RGB(RGB_R, RGB_G, RGB_B));
      end;

      function GetLightColor(BShadow : TColor) : TColor;
        var
         RGB_R,RGB_G,RGB_B : Byte;
      begin
         RGB_R := GetRValue (ColorToRGB (BShadow)) + GetRValue (ColorToRGB (BShadow)) div 5;
         RGB_G := GetGValue (ColorToRGB (BShadow)) + GetGValue (ColorToRGB (BShadow)) div 5;
         RGB_B := GetBValue (ColorToRGB (BShadow)) + GetBValue (ColorToRGB (BShadow)) div 5;
         if RGB_R > $ff then RGB_R := $ff;
         if RGB_G > $ff then RGB_G := $ff;
         if RGB_B > $ff then RGB_B := $ff;
         Result := TColor(RGB(RGB_R, RGB_G, RGB_B));
      end;
     var
        i : integer;
        bmpTmp : TBitmap;
     begin
       RW := Width + 8;
       RH := Height + 8;
       FFrame := TBitmap.Create;
       FFrame.Width := Width;
       FFrame.Height := Height;
       FFrame.Canvas.Pen.Style := psClear;
       if FStyle = bsLowered then
         begin
           clrFrom := GetDarkColor(clBtnShadow);
           clrTo := clBtnHighlight;
           FFrame.Canvas.Brush.Color := clBtnShadow;
           FFrame.Canvas.FillRect(FFrame.Canvas.ClipRect);
         end
       else
         begin
           clrTo := GetDarkColor(clBtnShadow);
           clrFrom := GetLightColor(clBtnFace);
           FFrame.Canvas.Brush.Color := clBtnHighlight;
           FFrame.Canvas.FillRect(FFrame.Canvas.ClipRect);
         end;
       bmpTmp := TBitmap.Create;
       bmpTmp.Width := Width;
       bmpTmp.Height := Height;
       bmpTmp.Canvas.Brush.Color := clLime;
       bmpTmp.Canvas.FillRect(bmpTmp.Canvas.ClipRect);
       bmpTmp.Canvas.Brush.Color := clRed;
       bmpTmp.Canvas.Pen.Style := psClear;
       case FBType of
          btRoundRect, btDblRRect, btInsetRRect:
                 begin
                   if Width >= Height then
                   bmpTmp.Canvas.RoundRect(0, 0, Width, Height, Height, Height)
                   else
                   bmpTmp.Canvas.RoundRect(0, 0, Width, Height, Width, Width);
                 end;
          btEllipse, btDblEllipse, btInsetEllipse:
                 begin
                   bmpTmp.Canvas.Ellipse(0, 0, Width, Height);
                 end;
          btChord:
                 begin
                   bmpTmp.Canvas.Chord( 0, 0, Width, Height + 3*Height div 4, Width, Height, 0, Height);
                 end;
        end;
       RGBFromR := GetRValue (ColorToRGB (ClrFrom));
       RGBFromG := GetGValue (ColorToRGB (ClrFrom));
       RGBFromB := GetBValue (ColorToRGB (ClrFrom));
       RGBDiffR := GetRValue (ColorToRGB (ClrTo)) - RGBFromR;
       RGBDiffG := GetGValue (ColorToRGB (ClrTo)) - RGBFromG;
       RGBDiffB := GetBValue (ColorToRGB (ClrTo)) - RGBFromB;

       for i := 0 to $80 do
          begin
            FFrame.Canvas.Brush.Color := TColor(RGB (RGBFromR + MulDiv(2*i,RGBDiffR,255),
                                      RGBFromG + MulDiv(2*i,RGBDiffG,255),
                                      RGBFromB + MulDiv(2*i,RGBDiffB,255)));
            PX := MulDiv (2*i,((7*RW) div 8),256);
            PY := MulDiv (2*i+1,((7*RH) div 8),256);
            PX := PX - 4;
            PY := PY - 4;
            KX := RW - (PX  div 9);
            KY := RH - (PY div 9);
            if FFrame.Canvas.Brush.Color = clBlack then Break;
            if (KX < RW div 8) or (KY < RH div 8) then Break;
            FFrame.Canvas.Rectangle (PX, PY, RW, RH)
          end;
          DrawTransparentBitmap(FFrame.Canvas, bmpTmp, 0, 0, clRed);
          bmpTmp.Free;
     end;

   procedure DrawBitmapUp;
     var
       i : integer;
       tmpUp : TBitmap;
     begin
     tmpUp := TBitmap.Create;
     case FBType of
        btRoundRect, btEllipse, btDblEllipse, btChord,
        btDblRRect, btInsetEllipse, btInsetRRect:
          begin
            RW := Width - 4;
            RH := Height - 4;
          end;
        btRect, btInsetRect:
          begin
           RW := Width;
           RH := Height;
          end;
       end;
       if FMSelect then
          begin
            RW := Width - 2;
            RH := Height - 2;
          end;
      case FBType of
        btDblEllipse, btInsetEllipse:
          begin
            RWS := RW - RW div 7;
            RHS := RH - RH div 7;
          end;
        btDblRRect, btInsetRRect:
          begin
            if RW >= RH then
              begin
               RWS := RW - RW div 9;
               RHS := RH - RH div 7;
              end
            else
              begin
               RWS := RW - RW div 7;
               RHS := RH - RH div 9;
              end;
          end;
         btInsetRect:
            if RW <= RH then
              begin
               RWS := RW - RW div 7;
               RHS := RH - RW div 7;
              end
            else
              begin
               RHS := RH - RH div 7;
               RWS := RW - RH div 7;
              end;
       end;
       tmpUp.Width := RW;
       tmpUp.Height := RH;
       FUp.Width := Width;
       FUp.Height := Height;
       FUp.Canvas.Pen.Style := psClear;
       FUp.Canvas.Brush.Color := clLime;
       FUp.Canvas.FillRect(FUp.Canvas.ClipRect);
       if FFType = ftBlackness then
          begin
            clrFrom := clBlack;
            clrTo := clWhite;
            RGBFromR := GetRValue (ColorToRGB (ClrFrom));
            RGBFromG := GetGValue (ColorToRGB (ClrFrom));
            RGBFromB := GetBValue (ColorToRGB (ClrFrom));
            RGBDiffR := GetRValue (ColorToRGB (FStart));
            RGBDiffG := GetGValue (ColorToRGB (FStart));
            RGBDiffB := GetBValue (ColorToRGB (FStart));
          end
       else
          begin
            if FFType = ftWhiteness then
               begin
                  clrFrom := FStart;
                  clrTo := FEnd;
               end
            else
               begin
                  clrFrom := FStart;
                  clrTo := FLight;
               end;
            RGBFromR := GetRValue (ColorToRGB (ClrFrom));
            RGBFromG := GetGValue (ColorToRGB (ClrFrom));
            RGBFromB := GetBValue (ColorToRGB (ClrFrom));
            RGBDiffR := GetRValue (ColorToRGB (ClrTo)) - RGBFromR;
            RGBDiffG := GetGValue (ColorToRGB (ClrTo)) - RGBFromG;
            RGBDiffB := GetBValue (ColorToRGB (ClrTo)) - RGBFromB;
          end;

       tmpUp.Canvas.Pen.Style := psClear;
       tmpUp.Canvas.Brush.Color := clLime;
       tmpUp.Canvas.FillRect(tmpUp.Canvas.ClipRect);

       for i := 0 to $80 do
          begin
            tmpUp.Canvas.Brush.Color := TColor(RGB (RGBFromR + MulDiv(2*i,RGBDiffR,255),
                                      RGBFromG + MulDiv(2*i,RGBDiffG,255),
                                      RGBFromB + MulDiv(2*i,RGBDiffB,255)));
            if (FBType = btRoundRect) or (FBType = btDblRRect) or
               (FBType = btInsetRRect) then
               begin
                 PX := MulDiv (2*i,(RW div 6),256);
                 PY := MulDiv (2*i,(RH div 6),256);
                 KX := RW - MulDiv (2*i,((5*RW) div 6),256);
                 KY := RH - MulDiv (2*i+1,((5*RH) div 6),256);
                 if (KX < RW div 7) or (KY < RH div 7) then Break;
                 TargetRect := Rect(PX, PY, KX, KY);
                 with TargetRect do
                   begin
                      if RW >= RH then
                        tmpUp.Canvas.RoundRect (Left, Top, Right, Bottom, KY, KY)
                      else
                        tmpUp.Canvas.RoundRect (Left, Top, Right, Bottom, KX, KX)
                   end;
               end;
            if (FBType = btEllipse) or (FBType = btDblEllipse) or
               (FBType = btInsetEllipse) then
               begin
                 PX := MulDiv (2*i,(RW div 4),256);
                 PY := MulDiv (2*i,(RH div 4),256);
                 KX := RW - MulDiv (2*i,((3*RW) div 4),256);
                 KY := RH - MulDiv (2*i+1,((3*RH) div 4),256);
                 if (KX < RW div 6) or (KY < RH div 6) then Break;
                 TargetRect := Rect(PX, PY, KX, KY);
                 with TargetRect do
                 tmpUp.Canvas.Ellipse (Left, Top, Right, Bottom);
               end;
            if FBType = btRect then
               begin
                 PX := MulDiv (2*i,(RW div 4),256);
                 PY := MulDiv (2*i,(RH div 3),256);
                 KX := RW - MulDiv (2*i,((3*RW) div 4),256);
                 KY := RH - MulDiv (2*i+1,((2*RH) div 3),256);
                 if (KX < RW div 6) or (KY < RH div 6) then Break;
                 TargetRect := Rect(PX, PY, KX, KY);
                 with TargetRect do
                 tmpUp.Canvas.Rectangle (Left, Top, Right, Bottom);
                end;
            if FBType = btInsetRect then
               begin
                 PX := MulDiv (2*i,(RW div 7),256);
                 PY := MulDiv (2*i,(RH div 7),256);
                 KX := RW - MulDiv (2*i,((6*RW) div 7),256);
                 KY := RH - MulDiv (2*i+1,((6*RH) div 7),256);
                 if (KX < RW div 8) or (KY < RH div 8) then Break;
                 TargetRect := Rect(PX, PY, KX, KY);
                 with TargetRect do
                 tmpUp.Canvas.Rectangle (Left, Top, Right, Bottom);
                end;
            if FBType = btChord then
               begin
                 PX := MulDiv (2*i,(RW div 4),256);
                 PY := MulDiv (2*i,(RH div 3),256);
                 KX := RW - MulDiv (2*i,((3*RW) div 4),256);
                 KY := RH - MulDiv (2*i+1,((2*RH) div 3),256);
                 if (KX < RW div 6) or (KY < RH div 6) then Break;
                 TargetRect := Rect(PX, PY, KX, KY);
                 with TargetRect do
                 tmpUp.Canvas.Chord (Left, Top, Right, Bottom + 3*Bottom div 4, Right, Bottom, Left, Bottom);
               end;
          end;
          if FBType = btInsetRect then
             begin
              for i := 0 to $80 do
                 begin
                    tmpUp.Canvas.Brush.Color := TColor(RGB (RGBFromR + MulDiv(2*i,RGBDiffR,255),
                                      RGBFromG + MulDiv(2*i,RGBDiffG,255),
                                      RGBFromB + MulDiv(2*i,RGBDiffB,255)));
                    PX := MulDiv (2*i,((3*RWS) div 4),256);
                    PY := MulDiv (2*i+1,((3*RHS) div 4),256);
                    KX := RWS - (PX  div 3);
                    KY := RHS - (PY div 3);
                    if RW <= RH then
                       begin
                         PX := RW div 7 + PX - 1;
                         PY := RW div 7 + PY - 1;
                       end
                    else
                       begin
                         PX := RH div 7 + PX - 1;
                         PY := RH div 7 + PY - 1;
                       end;
                    if (PX > 6*RW div 8) or (PY > 5*RH div 7) then Break;
                    TargetRect := Rect(PX, PY, KX, KY);
                     with TargetRect do
                       begin
                          if RW > RH then
                            tmpUp.Canvas.RoundRect (Left, Top, Right, Bottom, KY div 4, KY div 4)
                          else
                            tmpUp.Canvas.RoundRect (Left, Top, Right, Bottom, KX div 4, KX div 4);
                       end;
                end;
           end;
       if (FBType = btDblEllipse) or (FBType = btInsetEllipse) or
          (FBType = btDblRRect) or (FBType = btInsetRRect) or (FBType = btInsetRect) then
          begin
          for i := 0 to $80 do
             begin
               tmpUp.Canvas.Brush.Color := TColor(RGB (RGBFromR + MulDiv(2*i,RGBDiffR,255),
                                      RGBFromG + MulDiv(2*i,RGBDiffG,255),
                                      RGBFromB + MulDiv(2*i,RGBDiffB,255)));
                if FBType = btDblRRect then
                   begin
                     PX := RW div 9 + MulDiv (2*i,(RWS div 4),256) + 1;
                     PY := RH div 7 + MulDiv (2*i,(RHS div 4),256) - 1;
                     KX := RWS - MulDiv (2*i,((3*RWS) div 4),256);
                     KY := RHS - MulDiv (2*i+1,((3*RHS) div 4),256);
                     if (KX < RWS div 3) or (KY < RHS div 3) then Break;
                     TargetRect := Rect(PX, PY, KX, KY);
                     with TargetRect do
                       begin
                          if RW > RH then
                            tmpUp.Canvas.RoundRect (Left, Top, Right, Bottom, KY, KY)
                          else
                            tmpUp.Canvas.RoundRect (Left, Top, Right, Bottom, KX, KX);
                       end;
                   end;
                if FBType = btDblEllipse then
                  begin
                    PX := RW div 7 + MulDiv (2*i,(RWS div 4),256) + 1;
                    PY := RH div 7 + MulDiv (2*i,(RHS div 4),256) - 1;
                    KX := RWS - MulDiv (2*i,((3*RWS) div 4),256);
                    KY := RHS - MulDiv (2*i+1,((3*RHS) div 4),256);
                    if (KX < RWS div 3) or (KY < RHS div 3) then Break;
                    TargetRect := Rect(PX, PY, KX, KY);
                    with TargetRect do
                    tmpUp.Canvas.Ellipse (Left, Top, Right, Bottom);
                  end;
               if FBType = btInsetEllipse then
                 begin
                    PX := MulDiv (2*i,((3*RWS) div 4),256);
                    PY := MulDiv (2*i+1,((3*RHS) div 4),256);
                    KX := RWS - (PX  div 3);
                    KY := RHS - (PY div 3);
                    PX := RW div 7 + PX + 1;
                    PY := RH div 7 + PY - 1;
                    if (PX > 5*RW div 7) or (PY > 5*RH div 7) then Break;
                    TargetRect := Rect(PX, PY, KX, KY);
                    with TargetRect do
                    tmpUp.Canvas.Ellipse (Left, Top, Right, Bottom);
                 end;
              if FBType = btInsetRRect then
                 begin
                    PX := MulDiv (2*i,((3*RWS) div 4),256);
                    PY := MulDiv (2*i+1,((3*RHS) div 4),256);
                    KX := RWS - (PX  div 3);
                    KY := RHS - (PY div 3);
                    PX := RW div 9 + PX + 1;
                    PY := RH div 7 + PY - 1;
                    if (PX > 6*RW div 8) or (PY > 5*RH div 7) then Break;
                    TargetRect := Rect(PX, PY, KX, KY);
                     with TargetRect do
                       begin
                          if RW > RH then
                            tmpUp.Canvas.RoundRect (Left, Top, Right, Bottom, KY, KY)
                          else
                            tmpUp.Canvas.RoundRect (Left, Top, Right, Bottom, KX, KX);
                       end;
                 end;
             end;
          end;
         case FBType of
            btEllipse, btRoundRect, btDblEllipse, btChord,
            btInsetEllipse, btDblRRect, btInsetRRect:
               begin
                 if FMSelect = False then
                    begin
                      DrawTransparentBitmap(FUp.Canvas, FFrame, 0, 0, clLime);
                      DrawTransparentBitmap(FUp.Canvas, tmpUp, 2, 2, clLime);
                    end
                 else DrawTransparentBitmap(FUp.Canvas, tmpUp, 1, 1, clLime);
               end;
            btRect, btInsetRect:
               begin
                 DrawTransparentBitmap(FUp.Canvas, tmpUp, 0, 0, clLime);
               end;
         end;
         tmpUp.Free;
     end;

   procedure DrawBitmapDown;
     var
      i : integer;
      tmpDown : TBitmap;
     begin
     tmpDown := TBitmap.Create;
     case FBType of
        btRoundRect, btEllipse, btDblEllipse, btChord,
        btDblRRect, btInsetEllipse, btInsetRRect:
           begin
             RW := Width - 6;
             RH := Height - 6;
             if FMSelect then
               begin
                 RW := Width - 4;
                 RH := Height - 4;
               end;
              tmpDown.Width := RW;
              tmpDown.Height := RH;
            end;
        btRect, btInsetRect:
          begin
           RW := Width;
           RH := Height;
           if FMSelect then
             begin
               RW := Width - 4;
               RH := Height - 4;
             end;
           tmpDown.Width := RW;
           tmpDown.Height := RH;
          end;
       end;
      case FBType of
        btDblEllipse, btInsetEllipse:
          begin
            RWS := RW - RW div 7;
            RHS := RH - RH div 7;
          end;
        btDblRRect, btInsetRRect:
          begin
            if RW >= RH then
              begin
               RWS := RW - RW div 9;
               RHS := RH - RH div 7;
              end
            else
              begin
               RWS := RW - RW div 7;
               RHS := RH - RH div 9;
              end;
          end;
         btInsetRect:
            if RW <= RH then
              begin
               RWS := RW - RW div 7;
               RHS := RH - RW div 7;
              end
            else
              begin
               RHS := RH - RH div 7;
               RWS := RW - RH div 7;
              end;
       end;
       FDown.Width := Width;
       FDown.Height := Height;
       FDown.Canvas.Pen.Style := psClear;
       FDown.Canvas.Brush.Color := clLime;
       FDown.Canvas.FillRect(FDown.Canvas.ClipRect);
       if FFType = ftBlackness then
          begin
            clrFrom := clBlack;
            clrTo := clWhite;
            RGBFromR := GetRValue (ColorToRGB (ClrFrom));
            RGBFromG := GetGValue (ColorToRGB (ClrFrom));
            RGBFromB := GetBValue (ColorToRGB (ClrFrom));
            RGBDiffR := GetRValue (ColorToRGB (FStart));
            RGBDiffG := GetGValue (ColorToRGB (FStart));
            RGBDiffB := GetBValue (ColorToRGB (FStart));
          end
       else
          begin
            if FFType = ftWhiteness then
               begin
                  clrFrom := FStart;
                  clrTo := FEnd;
               end
            else
               begin
                  clrFrom := FStart;
                  clrTo := FLight;
               end;
            RGBFromR := GetRValue (ColorToRGB (ClrFrom));
            RGBFromG := GetGValue (ColorToRGB (ClrFrom));
            RGBFromB := GetBValue (ColorToRGB (ClrFrom));
            RGBDiffR := GetRValue (ColorToRGB (ClrTo)) - RGBFromR;
            RGBDiffG := GetGValue (ColorToRGB (ClrTo)) - RGBFromG;
            RGBDiffB := GetBValue (ColorToRGB (ClrTo)) - RGBFromB;
          end;

       tmpDown.Canvas.Pen.Style := psClear;
       tmpDown.Canvas.Brush.Color := clLime;
       tmpDown.Canvas.FillRect(tmpDown.Canvas.ClipRect);

       for i := 0 to $80 do
         begin
            tmpDown.Canvas.Brush.Color := TColor(RGB (RGBFromR + MulDiv(2*i,RGBDiffR,255),
                                      RGBFromG + MulDiv(2*i,RGBDiffG,255),
                                      RGBFromB + MulDiv(2*i,RGBDiffB,255)));
            PX := MulDiv (2*i,RW,256);
            PY := MulDiv (2*i,RH,256);
            KX := RW - MulDiv (2*i,RW,256);
            KY := RH - MulDiv (2*i+1,RH,256);
            if (KX < RW div 2) or (KY < RH div 2) then Break;
            TargetRect := Rect(PX, PY, KX, KY);
            if (FBType = btRoundRect) or (FBType = btDblRRect) then
               begin
                 with TargetRect do
                   begin
                      if RW >= RH then
                        tmpDown.Canvas.RoundRect (Left, Top, Right, Bottom, KY, KY)
                      else
                        tmpDown.Canvas.RoundRect (Left, Top, Right, Bottom, KX, KX);
                   end;
               end;
            if (FBType = btEllipse) or (FBType = btDblEllipse) then
               begin
                 with TargetRect do
                 tmpDown.Canvas.Ellipse (Left, Top, Right, Bottom);
               end;
            if (FBType = btRect) or (FBType = btInsetRect) then
               begin
                 with TargetRect do
                 tmpDown.Canvas.Rectangle (Left, Top, Right, Bottom)
               end;
            if FBType = btChord then
               begin
                 with TargetRect do
                 tmpDown.Canvas.Chord (Left, Top, Right, Bottom + 3*Bottom div 4, Right, Bottom, Left, Bottom);
               end;
         end;
         if FBType = btInsetRect then
            begin
            for i := 0 to $80 do
              begin
                 tmpDown.Canvas.Brush.Color := TColor(RGB (RGBFromR + MulDiv(2*i,RGBDiffR,255),
                                      RGBFromG + MulDiv(2*i,RGBDiffG,255),
                                      RGBFromB + MulDiv(2*i,RGBDiffB,255)));
                 PX := MulDiv (2*i,((3*RWS) div 4),256);
                 PY := MulDiv (2*i+1,((3*RHS) div 4),256);
                 KX := RWS - (PX  div 3);
                 KY := RHS - (PY div 3);
                 if RW <= RH then
                    begin
                      PX := RW div 7 + PX - 1;
                      PY := RW div 7 + PY - 1;
                    end
                 else
                    begin
                      PX := RH div 7 + PX - 1;
                      PY := RH div 7 + PY - 1;
                    end;
                 if (PX > 6*RW div 8) or (PY > 5*RH div 7) then Break;
                 TargetRect := Rect(PX, PY, KX, KY);
                 with TargetRect do
                   begin
                     if RW >= RH then
                        tmpDown.Canvas.RoundRect (Left, Top, Right, Bottom, KY div 4, KY div 4)
                     else
                        tmpDown.Canvas.RoundRect (Left, Top, Right, Bottom, KX div 4, KX div 4);
                   end;
             end;
      end;
      if (FBType = btDblRRect) or (FBType = btDblEllipse) then
         begin
            for i := 0 to $80 do
              begin
                 tmpDown.Canvas.Brush.Color := TColor(RGB (RGBFromR + MulDiv(2*i,RGBDiffR,255),
                                      RGBFromG + MulDiv(2*i,RGBDiffG,255),
                                      RGBFromB + MulDiv(2*i,RGBDiffB,255)));
                 PX := MulDiv (2*i,((3*RWS) div 4),256);
                 PY := MulDiv (2*i+1,((3*RHS) div 4),256);
                 KX := RWS - (PX  div 3);
                 KY := RHS - (PY div 3);
                 PX := RW div 9 + PX + 1;
                 PY := RH div 7 + PY - 1;
                 if (PX > 5*RW div 7) or (PY > 5*RH div 7) then Break;
                 TargetRect := Rect(PX, PY, KX, KY);
                 with TargetRect do
                   begin
                      if FBType = btDblRRect then
                         begin
                           if RW >= RH then
                             tmpDown.Canvas.RoundRect (Left, Top, Right, Bottom, KY, KY)
                           else
                              tmpDown.Canvas.RoundRect (Left, Top, Right, Bottom, KX, KX);
                         end;
                      if FBType = btDblEllipse then
                         begin
                            tmpDown.Canvas.Ellipse (Left, Top, Right, Bottom);
                         end;
                   end;
             end;
      end;
      if (FBType = btInsetRRect) or (FBType = btInsetEllipse) then
         begin
            for i := 0 to $80 do
               begin
                  tmpDown.Canvas.Brush.Color := TColor(RGB (RGBFromR + MulDiv(2*i,RGBDiffR,255),
                                      RGBFromG + MulDiv(2*i,RGBDiffG,255),
                                      RGBFromB + MulDiv(2*i,RGBDiffB,255)));
                  if FBType = btInsetRRect then
                     begin
                        PX := MulDiv (8*i,(RW div 3),256);
                        PY := MulDiv (8*i,(RH div 3),256);
                        KX := RW - MulDiv (8*i,((2*RW) div 3),256);
                        KY := RH - MulDiv (8*i+1,((2*RH) div 3),256);
                        if (KX < RW div 2) or (KY < RH div 2) then Break;
                        TargetRect := Rect(PX, PY, KX, KY);
                         with TargetRect do
                           begin
                             if RW >= RH then
                               tmpDown.Canvas.RoundRect (Left, Top, Right, Bottom, KY, KY)
                             else
                               tmpDown.Canvas.RoundRect (Left, Top, Right, Bottom, KX, KX);
                           end;
                    end;
                  if FBType = btInsetEllipse then
                     begin
                       PX := MulDiv (2*i,(RW div 3),256);
                       PY := MulDiv (2*i,(RH div 3),256);
                       KX := RW - MulDiv (2*i,((2*RW) div 3),256);
                       KY := RH - MulDiv (2*i+1,((2*RH) div 3),256);
                       if (KX < RW div 2) or (KY < RH div 2) then Break;
                       TargetRect := Rect(PX, PY, KX, KY);
                        with TargetRect do
                       tmpDown.Canvas.Ellipse (Left, Top, Right, Bottom);
                     end;
               end;
            for i := 0 to $80 do
               begin
                  tmpDown.Canvas.Brush.Color := TColor(RGB (RGBFromR + MulDiv(2*i,RGBDiffR,255),
                                      RGBFromG + MulDiv(2*i,RGBDiffG,255),
                                      RGBFromB + MulDiv(2*i,RGBDiffB,255)));
                   if FBType = btInsetEllipse then
                      begin
                        PX := MulDiv (2*i,((3*RWS) div 4),256);
                        PY := MulDiv (2*i+1,((3*RHS) div 4),256);
                        KX := RWS - (PX  div 3);
                        KY := RHS - (PY div 3);
                        PX := RW div 7 + PX + 1;
                        PY := RH div 7 + PY - 1;
                        if (PX > 5*RW div 7) or (PY > 5*RH div 7) then Break;
                        TargetRect := Rect(PX, PY, KX, KY);
                        with TargetRect do
                        tmpDown.Canvas.Ellipse (Left, Top, Right, Bottom);
                      end;
                   if FBType = btInsetRRect then
                      begin
                        PX := MulDiv (2*i,((3*RWS) div 4),256);
                        PY := MulDiv (2*i+1,((3*RHS) div 4),256);
                        KX := RWS - (PX  div 3);
                        KY := RHS - (PY div 3);
                        PX := RW div 9 + PX + 1;
                        PY := RH div 7 + PY - 1;
                        if (PX > 6*RW div 8) or (PY > 5*RH div 7) then Break;
                        TargetRect := Rect(PX, PY, KX, KY);
                         with TargetRect do
                           begin
                             if RW >= RH then
                               tmpDown.Canvas.RoundRect (Left, Top, Right, Bottom, KY, KY)
                             else
                               tmpDown.Canvas.RoundRect (Left, Top, Right, Bottom, KX, KX);
                           end;
                      end;
               end;
         end;
         case FBType of
            btEllipse, btRoundRect, btDblEllipse, btChord,
            btInsetEllipse, btDblRRect, btInsetRRect:
               begin
                 if FMSelect = False then
                   begin
                      DrawTransparentBitmap(FDown.Canvas, FFrame, 0, 0, clLime);
                   end;
                 DrawTransparentBitmap(FDown.Canvas, tmpDown, 3, 3, clLime);
                 FFrame.Free;
               end;
            btRect, btInsetRect:
               begin
                 DrawTransparentBitmap(FDown.Canvas, tmpDown, 0, 0, clLime);
               end;
         end;
         tmpDown.Free;
     end;


begin
   if csLoading in ComponentState then exit;
   if FBType <> btRect then DrawBitmapFrame;
   DrawBitmapUp;
   DrawBitmapDown;
   FBMP := True;
end;

procedure THS3dButton.SetBType(AType : TButtonType);
begin
   if AType <> FBType then
     begin
       FBType := AType;
       DrawBitmaps;
       Invalidate;
     end;
end;

procedure THS3dButton.SetFType(AType : TFaceType);
begin
   if AType <> FFType then
     begin
       FFType := AType;
       DrawBitmaps;
       Invalidate;
     end;
end;

procedure THS3DButton.MouseDown(Button : TMouseButton; Shift : TShiftState; X, Y : Integer);
begin
    if Button = mbLeft then
      begin
       FDwn := True;
       Invalidate;
      end;
    inherited MouseDown(Button, Shift, X, Y);
end;

procedure THS3DButton.MouseUp(Button : TMouseButton; Shift : TShiftState; X, Y : Integer);
begin
    if Button = mbLeft then
      begin
       FDwn := False;
       Invalidate;
      end;
    inherited MouseUp(Button, Shift, X, Y);
end;

procedure THS3DButton.SetCaption(ACaption : string);
begin
   if ACaption <> FCaption then
     begin
        FCaption := ACaption;
        Invalidate;
     end;
end;

procedure THS3DButton.SetPicture(APicture : TPicture);
begin
    FPicture.Assign(APicture);
    Invalidate;
end;

procedure THS3DButton.DrawGlyphAndCaption(Canvas : TCanvas);
 var
   ICO : TIcon;
   DBMP, BMP, BMP1 : TBitmap;
   WMF : TMetafile;
   RectWidth, RectHeight : Integer;
   DX, DY : Integer;
   ClrBack : LongInt;
begin
    Canvas.Brush.Style := bsClear;
    if FPicture.Graphic = nil then
       begin
         if FCaption <> '' then
            begin
              if Enabled = False then
                 begin
                   Canvas.Font.Color := clWhite;
                   Canvas.TextOut((Width - Canvas.TextWidth(FCaption)) div 2 + 1,
                  (Height - Canvas.TextHeight(Fcaption)) div 2 + 1, FCaption);
                   Canvas.Font.Color := clBtnShadow;
                   Canvas.TextOut((Width - Canvas.TextWidth(FCaption)) div 2,
                  (Height - Canvas.TextHeight(Fcaption)) div 2, FCaption);
                 end
              else
                 begin
                   if FMSelect and FMouse then Canvas.Font.Color := FSColor
                   else Canvas.Font.Color := Font.Color;
                   Canvas.TextOut((Width - Canvas.TextWidth(FCaption)) div 2,
                  (Height - Canvas.TextHeight(Fcaption)) div 2, FCaption);
                 end;
            end;
         Exit;
       end
    else
       begin
         if FPicture.Graphic is TBitmap then
            begin
                BMP := FPicture.Bitmap;
                DBMP := CreateDisabledBitmap(BMP, clBlack);
            end
         else
            begin
              if FPicture.Graphic is TIcon then
                 begin
                   BMP := GetBitmapFromIcon(FPicture.Icon, clBtnShadow, FPicture.Icon.Width);
                   DBMP := CreateDisabledBitmap(BMP, clBlack or clNavy);
                 end;
            end;
         if FCaption <> '' then
            begin
               RectWidth := BMP.Width + Canvas.TextWidth(FCaption) + 1;
               RectHeight := BMP.Height + Canvas.TextHeight(FCaption) + 1;
               case FGLayout of
                      glGlyphTop :
                         begin
                           DX := (Width - BMP.Width) div 2;
                           DY := (Height - RectHeight) div 2;
                           if Enabled = False then
                              begin
                                 Canvas.Font.Color := clWhite;
                                 Canvas.TextOut((Width - Canvas.TextWidth(FCaption)) div 2 + 1,
                                 DY + BMP.Height + 2, FCaption);
                                 Canvas.Font.Color := clBtnShadow;
                                 Canvas.TextOut((Width - Canvas.TextWidth(FCaption)) div 2,
                                 DY + BMP.Height + 1, FCaption);
                              end
                           else
                              begin
                                 if FMSelect and FMouse then Canvas.Font.Color := FSColor
                                 else Canvas.Font.Color := Font.Color;
                                 Canvas.TextOut((Width - Canvas.TextWidth(FCaption)) div 2,
                                 DY + BMP.Height + 1, FCaption);
                              end;
                         end;
                   glGlyphBottom :
                     begin
                       DX := (Width - BMP.Width) div 2;
                       DY := (Height - RectHeight) div 2 + (Canvas.TextHeight(FCaption) + 1);
                       if Enabled = False then
                          begin
                            Canvas.Font.Color := clWhite;
                            Canvas.TextOut((Width - Canvas.TextWidth(FCaption)) div 2 - 1,
                            DY - (Canvas.TextHeight(FCaption)), FCaption);
                            Canvas.Font.Color := clBtnShadow;
                            Canvas.TextOut((Width - Canvas.TextWidth(FCaption)) div 2,
                            DY - (Canvas.TextHeight(FCaption) + 1), FCaption);
                          end
                        else
                          begin
                            if FMSelect and FMouse then Canvas.Font.Color := FSColor
                            else Canvas.Font.Color := Font.Color;
                            Canvas.TextOut((Width - Canvas.TextWidth(FCaption)) div 2,
                            DY - (Canvas.TextHeight(FCaption) + 1), FCaption);
                          end;
                     end;
                   glGlyphLeft :
                     begin
                       DX := (Width - RectWidth) div 2;
                       DY := (Height - BMP.Height) div 2 ;
                       if Enabled = False then
                          begin
                            Canvas.Font.Color := clWhite;
                            Canvas.TextOut(DX + BMP.Width + 2, (Height - Canvas.TextHeight(FCaption)) div 2 + 1, FCaption);
                            Canvas.Font.Color := clBtnShadow;
                            Canvas.TextOut(DX + BMP.Width + 1, (Height - Canvas.TextHeight(FCaption)) div 2, FCaption);
                          end
                        else
                          begin
                            if FMSelect and FMouse then Canvas.Font.Color := FSColor
                            else Canvas.Font.Color := Font.Color;
                            Canvas.TextOut(DX + BMP.Width + 1, (Height - Canvas.TextHeight(FCaption)) div 2, FCaption);
                          end;
                     end;
                   glGlyphRight :
                     begin
                       DX := (Width - RectWidth) div 2 + Canvas.TextWidth(FCaption) + 1;
                       DY := (Height - BMP.Height) div 2 ;
                       if Enabled = False then
                          begin
                            Canvas.Font.Color := clWhite;
                            Canvas.TextOut((Width - RectWidth) div 2 + 1, (Height - Canvas.TextHeight(FCaption)) div 2 + 1,
                            FCaption);
                            Canvas.Font.Color := clBtnShadow;
                            Canvas.TextOut((Width - RectWidth) div 2, (Height - Canvas.TextHeight(FCaption)) div 2,
                            FCaption);
                          end
                        else
                          begin
                            if FMSelect and FMouse then Canvas.Font.Color := FSColor
                            else Canvas.Font.Color := Font.Color;
                            Canvas.TextOut((Width - RectWidth) div 2, (Height - Canvas.TextHeight(FCaption)) div 2, FCaption);
                          end;
                     end;
                 end;
              end
           else
              begin
                 DX := (Width - BMP.Width) div 2;
                 DY := (Height - BMP.Height) div 2;
              end;
           if Enabled = True then
              begin
                if BMP.Canvas.Pixels[0,BMP.Height - 1] = clWhite then
                ChangeBitmapColor(BMP, clWhite, clLime);
                DrawTransparentBitmap(Canvas, BMP, DX, DY, BMP.Canvas.Pixels[0,BMP.Height - 1]);
              end
           else
           DrawTransparentBitmap(Canvas, DBMP, DX, DY, DBMP.Canvas.Pixels[0,DBMP.Height - 1]);
           DBMP.Free;
    end;
end;

procedure THS3DButton.SetLayout(ALayout : TGlyphLayout);
begin
   if ALayout <> FGLayout then
     begin
       FGLayout := ALayout;
       Invalidate;
     end;
end;

procedure THS3DButton.SetBounds(aLeft, aTop, aWidth, aHeight: Integer);
begin
  inherited SetBounds(aLeft, aTop, aWidth, aHeight);
  if (aWidth <> FWidth) or (aHeight <> FHeight) then
    begin
    if not (csLoading in ComponentState) then
      begin
        DrawBitmaps;
        Invalidate;
      end;
     FWidth := aWidth;
     FHeight := aHeight;
   end;
end;

procedure THS3DButton.SetFStyle(AStyle : TBevelStyle);
begin
   if AStyle <> FStyle then
     begin
       FStyle := AStyle;
       if FMSelect = False then
         begin
          DrawBitmaps;
          Invalidate;
         end;
     end;
end;

procedure THS3DButton.DrawFace;
var
  bmpBuff : TBitmap;
  rct : TRect;
begin
   bmpBuff := TBitmap.Create;
   bmpBuff.Width := Width;
   bmpBuff.Height := Height;
   bmpBuff.Canvas.Brush.Color := clLime;
   bmpBuff.Canvas.FillRect(bmpBuff.Canvas.ClipRect);
   rct := Bounds(0, 0, Width, Height);
   with bmpBuff.Canvas do
     begin
       Font := Self.Font;
       case FBType of
          btRoundRect, btEllipse, btDblEllipse, btDblRRect,
          btInsetEllipse, btInsetRRect, btChord:
             begin
                   if FDwn = True then
                      begin
                        DrawTransparentBitmap(bmpBuff.Canvas, FDown, 0, 0, clLime);
                      end
                   else
                      begin
                        DrawTransparentBitmap(bmpBuff.Canvas, FUp, 0, 0, clLime);
                       end;
             end;
          btRect, btInsetRect:
             begin
               if FDwn = True then
                  begin
                    DrawTransparentBitmap(bmpBuff.Canvas, FDown, 0, 0, clLime);
                    if FStyle = bsLowered then
                      Frame3D(bmpBuff.Canvas, rct, clBtnShadow, clBtnHighlight, 2)
                    else
                      Frame3D(bmpBuff.Canvas, rct, clBtnHighlight, clBtnShadow, 1);
                  end
               else
                  begin
                    DrawTransparentBitmap(bmpBuff.Canvas, FUp, 0, 0, clLime);
                    if FStyle = bsLowered then
                      Frame3D(bmpBuff.Canvas, rct, clBtnShadow, clBtnHighlight, 1)
                    else
                      Frame3D(bmpBuff.Canvas, rct, clBtnHighlight, clBtnShadow, 2);
                  end;
             end;
       end;
       DrawGlyphAndCaption(bmpBuff.Canvas);
     end;
     bmpBuff.Canvas.Brush.Style := bsClear;
     bmpBuff.Canvas.Pen.Color := clBtnFace;
     if (FBType = btEllipse) or (FBType = btDblEllipse) or (FBType = btInsetEllipse) then
        begin
          bmpBuff.Canvas.Arc(0, 0, Width, Height, Width div 2, Height, Width div 2, 0);
        end
     else
        begin
          if (FBType = btRoundRect) or (FBType = btDblRRect) or (FBType = btInsetRRect) then
             begin
               if Width > Height then
                   bmpBuff.Canvas.Arc(Width - Height, 0, Width, Height,
                   Width - Height div 2, Height, Width - Height div 2, 0)
               else
                  bmpBuff.Canvas.RoundRect( 0, 0, Width, Height, Width, Width);
             end;
        end;
     DrawTransparentBitmap(Canvas, bmpBuff, 0, 0, clLime);
     bmpBuff.Free;
end;


procedure THS3DButton.DrawSelectFace;
var
   bmpBuff : TBitmap;
begin
   bmpBuff := TBitmap.Create;
   bmpBuff.Width := Width;
   bmpBuff.Height := Height;
   bmpBuff.Canvas.Brush.Color := clLime;
   bmpBuff.Canvas.FillRect(bmpBuff.Canvas.ClipRect);
   with bmpBuff.Canvas do
     begin
       Font := Self.Font;
       case FBType of
          btRoundRect, btDblRRect, btInsetRRect:
             begin
               if FMouse and (FDwn = False) then
                  begin
                    Pen.Color := FSColor;
                    Pen.Width := 1;
                    Brush.Color := FSColor;
                    if Width > Height then
                      RoundRect(0, 0, Width + 1, Height + 1, Height + 1, Height + 1)
                    else
                      RoundRect(0, 0, Width + 1, Height + 1, Width + 1, Width + 1);
                  end;
               if FDwn = True then
                  begin
                     DrawTransparentBitmap(bmpBuff.Canvas, FDown, 2, 2, clLime);
                  end
               else
                  begin
                     DrawTransparentBitmap(bmpBuff.Canvas, FUp, 1, 1, clLime);
                  end;
             end;
          btEllipse, btDblEllipse, btInsetEllipse:
             begin
               if FMouse and (FDwn = False) then
                  begin
                    Pen.Color := FSColor;
                    Pen.Width := 1;
                    Brush.Color := FSColor;
                    Ellipse(0, 0, Width + 1, Height + 1);
                  end;
               if FDwn = True then
                 begin
                   DrawTransparentBitmap(bmpBuff.Canvas, FDown, 2, 2, clLime);
                 end
               else
                 begin
                   DrawTransparentBitmap(bmpBuff.Canvas, FUp, 1, 1, clLime);
                 end;
              end;
          btRect, btInsetRect:
             begin
               if FMouse and (FDwn = False) then
                  begin
                    Pen.Color := FSColor;
                    Pen.Width := 1;
                    Brush.Color := FSColor;
                    Rectangle(0, 0, Width, Height);
                  end;
               if FDwn = True then
                  begin
                     DrawTransparentBitmap(bmpBuff.Canvas, FDown, 2, 2, clLime);
                  end
               else
                  begin
                     DrawTransparentBitmap(bmpBuff.Canvas, FUp, 1, 1, clLime);
                  end;
             end;
          btChord:
             begin
               if FMouse and (FDwn = False) then
                  begin
                    Pen.Color := FSColor;
                    Pen.Width := 1;
                    Brush.Color := FSColor;
                    Chord( 0, 0, Width, Height + 3*Height div 4, Width, Height, 0, Height);
                  end;
               if FDwn = True then
                  begin
                     DrawTransparentBitmap(bmpBuff.Canvas, FDown, 2, 2, clLime);
                  end
               else
                  begin
                     DrawTransparentBitmap(bmpBuff.Canvas, FUp, 1, 1, clLime);
                  end;
             end;
       end;
       DrawGlyphAndCaption(bmpBuff.Canvas);
     end;
     DrawTransparentBitmap(Canvas, bmpBuff, 0, 0, clLime);
     bmpBuff.Free;
end;

procedure THS3DButton.SetMouseSelect(ASelect: Boolean);
begin
   if ASelect = FMSelect then Exit;
   FMSelect := ASelect;
   DrawBitmaps;
   Invalidate;
end;

procedure THS3DButton.CMMouseEnter(var msg: TMessage);
begin
    FMouse := True;
    if FMSelect and (Enabled = True) then Invalidate;
end;

procedure THS3DButton.CMMouseLeave(var msg: TMessage);
begin
    FMouse := False;
    if FMSelect and (Enabled = True) then Invalidate;
end;

procedure THS3DButton.SetLight(AColor : TColor);
begin
   if AColor <> FLight then
    begin
       FLight := AColor;
       if FFType = ftTwoColors then
         begin
            DrawBitmaps;
            Invalidate;
         end;
    end;
end;

end.
