unit Hsdrvcmb;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, FileCtrl, HSUtils, HSgrphic;

type
  TComboType = (ctExplorerCombo, ctDriveCombo);

type
  THSDriveComboBox = class(TCustomComboBox)
  private
    { Private declarations }
  FDirectory : string;
  FDrive : string;
  FDrives : TStringList;
  FDriveIndex : integer;
  FDriveBmp : TList;
  FLevels : TStringList;
  FOnChange : TNotifyEvent;
  FComboType : TComboType;
  protected
    { Protected declarations }
    procedure BuildList; virtual;
    procedure CreateWnd; override;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure SetDirectory(ADirectory : string);
    procedure SetComboType(AComboType : TComboType);
    procedure Click; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Text;
    property Drive: string read FDrive write FDrive;
    property Directory : string read FDirectory write SetDirectory;
 published
    { Published declarations }
    property Color;
    property Ctl3D;
    property ComboType: TComboType read FComboType write SetComboType;
    property DragMode;
    property DragCursor;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property TabOrder;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
end;

procedure Register;

{$R HSDRVCMB}

implementation

procedure Register;
begin
  RegisterComponents('HS Controls', [THSDriveComboBox]);
end;

constructor THSDriveComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDrives := TStringList.Create;
  FLevels := TStringList.Create;
  FDriveBmp := TList.Create;
  Style := csOwnerDrawFixed;
  ItemHeight := 18;
  Sorted := True;
  Font.Name := 'MS Sans Serif';
  Font.Size := 9;
  FComboType := ctExplorerCombo;
  FDirectory := Copy(ExtractFilePath(Application.ExeName), 1, 3);
end;

destructor THSDriveComboBox.Destroy;
var
  i : integer;
  BMP : TBitMap;
begin
  FLevels.Free;
  for i := 0 to FDriveBmp.Count - 1 do
     begin
       BMP := FDriveBmp.Items[i];
       BMP.Free;
     end;
  FDrives.Free;
  FDriveBmp.Free;
  inherited Destroy;
end;

procedure THSDriveComboBox.CreateWnd;
begin
  inherited CreateWnd;
  BuildList;
  if FComboType = ctExplorerCombo then SetDirectory(FDirectory)
  else
    begin
      FDrive := FDrives.Strings[2];
      ItemIndex := 2;
    end;
end;

procedure THSDriveComboBox.BuildList;
var
  Letter: Char;
  DriveType : TDriveType;
  title : string;
  BMP : TBitMap;
begin
  FDriveIndex := - 1;
  for Letter := 'A' to 'Z' do begin
    DriveType := GuessDriveType(Letter);
    if DriveType <> dtNoDrive then begin
      FDriveIndex := FDriveIndex + 1;
      case DriveType of
        dtFloppy  : begin
                      title := Letter + ': [3 Floppy]';
                      BMP := TBitMap.Create;
                      BMP.Handle := LoadBitmap(hInstance, 'HSCMB_FLOPPY');
                    end;
        dtCDROM   : begin
                      title := Letter + ': [CDROM drive]';
                      BMP := TBitMap.Create;
                      BMP.Handle := LoadBitmap(hInstance, 'HSCMB_CDDRIVE');
                    end;
        dtFixed   : begin
                      title := Letter + ': [' + GetVolumeID(Letter) + ']';
                      BMP := TBitMap.Create;
                      BMP.Handle := LoadBitmap(hInstance, 'HSCMB_DRIVE');
                    end;
        dtNetwork : begin
                      title := Letter + ': [' + GetNetworkVolume(Letter) + ']';
                      BMP := TBitMap.Create;
                      BMP.Handle := LoadBitmap(hInstance, 'HSCMB_NETDRIVE');
                    end;
        dtRAM     : begin
                      title := Letter + ': [' + GetVolumeID(Letter) + ']';
                      BMP := TBitMap.Create;
                      BMP.Handle := LoadBitmap(hInstance, 'HSCMB_DRIVE');
                    end;
      end;
      Items.Add(lowercase(title));
      FDrives.Add(Letter + ':\');
      FLevels.Add(IntToStr(0));
      FDriveBmp.Add(BMP);
    end;
  end;
  DropDownCount := Items.Count - 1;
end;

procedure THSDriveComboBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  BMP : TBitMap;
  Offset : integer;
begin
    with Canvas do
      begin
         FillRect(Rect);
         BMP := FDriveBmp.Items[Index];
         if Pos('[', Items[index]) <> 0 then
           begin
{             DrawTransparentBitmap(Canvas, BMP, Rect.Left + 2, Rect.Top + 1,
             BMP.Canvas.Pixels[0, BMP.Height - 1]);}
             BrushCopy(Bounds(Rect.Left + 2, Rect.Top + 1, BMP.Width, BMP.Height), BMP,
                     Bounds(0, 0, BMP.Width, BMP.Height), BMP.Canvas.Pixels[0, BMP.Height - 1]);
             TextOut(Rect.left + 20, Rect.Top + 1, Items[Index]);
           end
         else
           begin
             Offset := StrToInt(FLevels.Strings[index]) * BMP.width;
{             DrawTransparentBitmap(Canvas, BMP, Rect.Left + 2, Rect.Top + 1,
             BMP.Canvas.Pixels[0, BMP.Height - 1]);}
             BrushCopy(Bounds(Rect.Left + 2 + Offset, Rect.Top + 1, BMP.Width, BMP.Height), BMP,
                     Bounds(0, 0, BMP.Width, BMP.Height), BMP.Canvas.Pixels[0, BMP.Height - 1]);
             TextOut(Rect.left + 20 + Offset, Rect.Top + 1, Items[Index]);
           end;
      end;
end;

procedure THSDriveComboBox.SetDirectory(ADirectory: string);
var
  IndexOfDrive,
  i, j, Level,
  Count : integer;
  CmbPath, Path, AppPath : string;
  BMP : TBitMap;
  Exists: Boolean;
begin
   if (FComboType = ctDriveCombo) or (FDirectory = '') then exit;
   Exists := false;
   Level := 0;
   Count := FDrives.Count - 1;
   for i := Count downto 0 do
      if Length(FDrives.Strings[i]) > 3 then
        begin
           Items.Delete(i);
           FDrives.Delete(i);
           FLevels.Delete(i);
           BMP := FDriveBmp.Items[i];
           BMP.Free;
           FDriveBmp.Delete(i);
        end
      else
        begin
           if ADirectory = FDrives.Strings[i] then
             begin
               Exists := true;
               IndexOfDrive := i;
             end;
        end;
   if Exists = true then
     begin
       FDirectory := ADirectory;
       ItemIndex := IndexOfDrive;
       exit;
     end;
   FDirectory := ADirectory;
   IndexOfDrive := FDrives.IndexOf(Copy(FDirectory, 1, 3)) + 1;
   if Copy(FDirectory, length(FDirectory) - 1, 1) = '\' then
      FDirectory := Copy(FDirectory, 0, length(FDirectory) - 1);
   CmbPath := Copy(FDirectory, 1, 3);
   AppPath := Copy(FDirectory, 4, length(FDirectory));
   if Pos('\', AppPath) <> 0 then
   Path := Copy(AppPath, 0, Pos('\', AppPath) - 1)
   else Path := AppPath;
   AppPath := Copy(AppPath, length(Path) + 2, length(AppPath));
   CmbPath := CmbPath + Path;
   FDrives.Insert(IndexOfDrive, CmbPath);
   Items.Insert(IndexOfDrive, lowercase(Path));
   Level := Level + 1;
   FLevels.Insert(IndexOfDrive, IntToStr(Level));
   BMP := TBitMap.Create;
   j:= Pos('\', AppPath);
   if (j = 0) and (AppPath = '') then
   BMP.Handle := LoadBitmap(hInstance, 'HSCMB_FOLDEROPEN')
   else
   BMP.Handle := LoadBitmap(hInstance, 'HSCMB_FOLDERCLOSE');
   FDriveBmp.Insert(IndexOfDrive, BMP);
   while j <> 0 do
     begin
        IndexOfDrive := IndexOfDrive + 1;
        Path := Copy(AppPath, 0, Pos('\', AppPath) - 1);
        AppPath := Copy(AppPath, length(Path) + 2, length(AppPath));
        CmbPath := CmbPath + '\' + Path;
        FDrives.Insert(IndexOfDrive, CmbPath);
        Items.Insert(IndexOfDrive, lowercase(Path));
        Level := Level + 1;
        FLevels.Insert(IndexOfDrive, IntToStr(Level));
        j:= Pos('\', AppPath);
        BMP := TBitMap.Create;
        if (j = 0) and (AppPath = '') then
        BMP.Handle := LoadBitmap(hInstance, 'HSCMB_FOLDEROPEN')
        else
        BMP.Handle := LoadBitmap(hInstance, 'HSCMB_FOLDERCLOSE');
        FDriveBmp.Insert(IndexOfDrive, BMP);
     end;
   if AppPath <> '' then
      begin
        IndexOfDrive := IndexOfDrive + 1;
        Path := AppPath;
        CmbPath := CmbPath + '\' + Path;
        FDrives.Insert(IndexOfDrive, CmbPath);
        Items.Insert(IndexOfDrive, lowercase(Path));
        Level := Level + 1;
        FLevels.Insert(IndexOfDrive, IntToStr(Level));
        BMP := TBitMap.Create;
        BMP.Handle := LoadBitmap(hInstance, 'HSCMB_FOLDEROPEN');
        FDriveBmp.Insert(IndexOfDrive, BMP);
      end;
   DropDownCount := Items.Count - 1;
   ItemIndex := IndexOfDrive;
end;

procedure THSDriveComboBox.Click;
begin
   if FComboType = ctExplorerCombo then
      begin
         SetDirectory(FDrives.Strings[ItemIndex]);
      end
   else
      begin
        FDrive := FDrives.Strings[ItemIndex];
      end;
   inherited Click;
end;

procedure THSDriveComboBox.SetComboType(AComboType : TComboType);
var
  Count, i : integer;
begin
   if (FComboType = ctExplorerCombo) and (AComboType = ctDriveCombo) then
      begin
        Count := FDrives.Count - 1;
        for i := Count downto 0 do
        if Length(FDrives.Strings[i]) > 3 then
           begin
             Items.Delete(i);
             FDrives.Delete(i);
             FLevels.Delete(i);
             FDriveBmp.Delete(i);
           end;
       end;
   FComboType := AComboType;
end;

initialization
DetectCompyterDrives;
end.
