unit Hsgrd;

interface

uses WinTypes, Messages, Classes, Graphics, Forms, Controls, ExtCtrls;

type
  TBevelStyle = (gbNone, gbLowered, gbRaised);
  TCellBevelStyle = (cbLowered, cbRaised);

type
  THSBevelGrid = class(TGraphicControl)
  private
    FCellGraphic : TList;
    FRect : TRect;
    FCellBrushStyle : TBrushStyle;
    FBorder : TBorderStyle;
    FColor,
    FBevelColor,
    FBorderColor,
    FCBorderColor  : TColor;
    FPicture : TPicture;
    FFrameWidth, FPrivWidth,
    FOldIndex, FNumCell,
    FCellIndex,
    FCellXSize, FCellYSize,
    FNumX, FNumY: Integer;
    FTransparent, FStretch : Boolean;
    FBevel : TBevelStyle;
    FCellDiff : Integer;
    FCBorder : TCellBevelStyle;
    procedure UpdateCellSizes(DoRepaint: Boolean);
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure SetNumX(ANumX : integer);
    procedure SetNumY(ANumY : integer);
    procedure SetBevel(AStyle : TBevelStyle);
    procedure SetCellBorder(AStyle : TCellBevelStyle);
    procedure SetTransparent(ATran : Boolean);
    procedure SetColor(AColor : TColor);
    procedure SetCellDiff(AWidth : integer);
    procedure SetPicture(APicture : TPicture);
    procedure SetCellWidth(AWidth : integer);
    procedure SetCellHeight(AHeight : integer);
    procedure SetBorder(ABorder : TBorderStyle);
    procedure SetBorderColor(AColor : TColor);
    procedure SetBevelColor(AColor : TColor);
    procedure SetCellBorderColor(AColor : TColor);
    procedure SetCellBrushStyle(AStyle : TBrushStyle);
    procedure SetFrameWidth(AWidth : Integer);
    procedure UpdateBounds;
    procedure DrawPicture(CellRect : TRect);
    procedure SetStretch(AStretch : Boolean);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh; virtual;
    procedure DrawCell(Cell: Integer; Focus : Boolean); virtual;
    procedure DrawCellGraphic(Cell: Integer; CellRect : TRect); virtual;
    function  CellAtPos(X, Y: Integer): TRect; virtual;
    procedure SetCellIndex(Index: Integer);
    function  RectAtPos(X, Y: Integer): TRect; virtual;
    property  Canvas;
    procedure MouseUp(Button : TMouseButton; Shift : TShiftState; X, Y : Integer); override;
    property  CellIndex : Integer read FCellIndex write SetCellIndex;
    property  CellWidth : integer read FCellXSize write SetCellWidth;
    property  CellHeight : integer read FCellYSize write SetCellHeight;
    property  CurrentRect : Trect read FRect write FRect;
    procedure CellGraphic(Index : Integer; AGraphic : TGraphic); virtual;
    property  CellCount: Integer read FNumCell write FNumCell;
  published
    property CellBrushStyle : TBrushStyle read FCellBrushStyle write SetCellBrushStyle;
    property BorderStyle: TBorderStyle read FBorder write SetBorder;
    property BevelColor: TColor read FBevelColor write SetBevelColor;
    property BorderColor: TColor read FBorderColor write SetBorderColor;
    property BevelStyle : TBevelStyle read FBevel write SetBevel;
    property CellBevelStyle : TCellBevelStyle read FCBorder write SetCellBorder;
    property CellBevelColor : TColor read FCBorderColor write SetCellBorderColor;
    property RowCount : Integer read FNumY write SetNumY;
    property ColCount : Integer read FNumX write SetNumX;
    property Transparent : Boolean read FTransparent write SetTransparent;
    property CellColor : TColor read FColor write SetColor;
    property CellFrameWidth : integer read FCellDiff write SetCellDiff;
    property Picture : TPicture read FPicture write SetPicture;
    property FrameWidth : integer read FFrameWidth write SetFrameWidth;
    property Stretch : Boolean read FStretch write SetStretch;
    property Font;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

uses SysUtils, WinProcs, Consts, StdCtrls;

procedure Register;
begin
 RegisterComponents('HS Controls', [THSBevelGrid]);
end;

constructor THSBevelGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle;
  FNumX := 1;
  FNumY := 1;
  FBevel := gbLowered;
  FCBorder := cbLowered;
  Color := clBtnFace;
  FColor := clBtnFace;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  FCellDiff := 1;
  FTransparent := true;
  FPicture := TPicture.Create;
  FBorder := bsNone;
  FBorderColor := clBlack;
  FCBorderColor := clBtnShadow;
  FBevelColor := clBtnShadow;
  FCellBrushStyle := bsClear;
  FCellIndex := 0;
  FOldIndex := -1;
  FFrameWidth := 5;
  FPrivWidth := 5;
  FCellGraphic := TList.Create;
  FNumCell := FNumX * FNumY;
  {FCellGraphic.Add(nil);}
  FStretch := False;
end;


destructor THSBevelGrid.Destroy;
begin
  FPicture.Free;
  FCellGraphic.Clear;
  FCellGraphic.Free;
  inherited Destroy;
end;

procedure THSBevelGrid.DrawCell(Cell: Integer; Focus : Boolean);
var
  WinTop, WinLeft: Integer;
  CellRect: TRect;
  PolygonPoints : array [0..3] of TPoint;
  OldColor : TColor;
begin
    OldColor := Canvas.Brush.Color;
    if FCellDiff = 0 then
       begin
         WinTop := (Cell div FNumX) * FCellYSize + FFrameWidth;
         WinLeft := (Cell mod FNumX) * FCellXSize + FFrameWidth;
       end
    else
        begin
         WinTop := (Cell div FNumX) * FCellYSize + FFrameWidth;
         WinLeft := (cell mod FNumX) * FCellXSize + FFrameWidth;
       end;
    CellRect := Bounds(WinLeft, WinTop, FCellXSize, FCellYSize);
    PolygonPoints[0].X := CellRect.Left;
    PolygonPoints[0].Y := CellRect.Top;
    PolygonPoints[1].X := CellRect.Right;
    PolygonPoints[1].Y := CellRect.Top;
    PolygonPoints[2].X := CellRect.Right;
    PolygonPoints[2].Y := CellRect.Bottom;
    PolygonPoints[3].X := CellRect.Left;
    PolygonPoints[3].Y := CellRect.Bottom;
    Canvas.Pen.Color := clBtnFace;
    with CellRect do
    if (FTransparent = true) or (FPicture <> nil) then
       Canvas.Polygon(PolygonPoints[0])
    else Canvas.Rectangle(Left, Top, Right, Bottom);
    InflateRect(CellRect, - FCellDiff, - FCellDiff);
    if FCBorder = cbLowered then
    Frame3D(Canvas, CellRect, FCBorderColor, clBtnHighlight, 1)
    else
    Frame3D(Canvas, CellRect, clBtnHighlight, FCBorderColor, 1);
    Canvas.Brush.Color := OldColor;
    if FTransparent = false then
      begin
        if Focus = false then Canvas.Brush.Color := FColor
        else Canvas.Brush.Color := clHighlight;
        Canvas.Brush.Style := bsSolid;
        Canvas.FillRect(CellRect);
      end;
    Canvas.Pen.Color := FCBorderColor;
    Canvas.Pen.Width := 1;
    Canvas.Pen.Style := psSolid;
    Canvas.Brush.Color := FCBorderColor;
    Canvas.Brush.Style := FCellBrushStyle;
    Canvas.Rectangle(CellRect.Left, CellRect.Top,
                     CellRect.Right, CellRect.Bottom);
    if FTransparent = false then
    if Cell < FCellGraphic.Count then
    if FCellGraphic.Items[Cell] <> nil then
       DrawCellGraphic(Cell, CellRect);
end;

procedure THSBevelGrid.Paint;
var
  Row, Col, wEntryIndex: Integer;
  DrawRect : TRect;
begin
  UpdateCellSizes(False);
  DrawRect := Bounds(0, 0, Width, Height);
  if FTransparent = false then
     begin
        Canvas.Brush.Color := clBtnFace;
        Canvas.Brush.Style := bsSolid;
        Canvas.FillRect(Canvas.ClipRect);
     end;
  if FPicture <> nil then
    begin
      InflateRect(DrawRect, - 1, - 1);
      DrawPicture(DrawRect);
    end;
  for Row := 0 to FNumY - 1 do
    begin
      for Col := 0 to FNumX - 1 do
         begin
           wEntryIndex := Row * FNumX + Col;
           DrawCell(wEntryIndex, False);
         end;
    end;
  if FBevel = gbRaised then
    Frame3D(Canvas, DrawRect, clBtnHighlight, FBevelColor, 1);
  if FBevel = gbLowered then
    Frame3D(Canvas, DrawRect, FBevelColor, clBtnHighlight, 1);
  if FBorder = bsSingle then
     begin
       Canvas.Pen.Color := FBorderColor;
       Canvas.Pen.Style := psSolid;
       Canvas.Pen.Width := 1;
       Canvas.Brush.Style := bsClear;
       Canvas.Rectangle(0, 0, Width, Height);
     end;
end;

function THSBevelGrid.CellAtPos(X, Y: Integer): TRect;
var
  Rect: TRect;
  STop, SLeft : integer;
begin
  if X > Width - 1 then X := Width - 1
  else if X < 0 then X := 0;
  if Y > Height - 1 then Y := Height - 1
  else if Y < 0 then Y := 0;
  SetCellIndex((Y div FCellYSize) * FNumX + (X div FCellXSize));
  STop := (FCellIndex div FNumX) * FCellXSize + FFrameWidth;
  SLeft := (FCellIndex mod FNumX) * FCellYSize + FFrameWidth;
  Rect := Bounds(SLeft, STop, FCellXSize, FCellYSize);
  InflateRect(Rect, - (FCellDiff + 3), - (FCellDiff + 3));
  Result := Rect;
end;

function THSBevelGrid.RectAtPos(X, Y: Integer): TRect;
var
  NumSquare, STop, SLeft : integer;
begin
  if X > Width - 1 then X := Width - 1
  else if X < 0 then X := 0;
  if Y > Height - 1 then Y := Height - 1
  else if Y < 0 then Y := 0;
  NumSquare := (Y div FCellYSize) * FNumX + (X div FCellXSize);
  STop := (NumSquare div FNumX) * FCellXSize;
  SLeft := (NumSquare mod FNumX) * FCellYSize;
  Result := Bounds(SLeft , STop, FCellXSize, FCellYSize);
end;

procedure THSBevelGrid.MouseUp(Button : TMouseButton; Shift : TShiftState; X, Y : Integer);
begin
    if Button = mbLeft then
      begin
        if FTransparent = false then
          begin
            if FOldIndex <> - 1 then
              begin
               DrawCell(FOldIndex, False);
              end;
            CellAtPos(X, Y);
            DrawCell(FCellIndex, True);
            FOldIndex := FCellIndex;
         end;
    end;
    inherited MouseUp(Button, Shift, X, Y);
end;

procedure THSBevelGrid.UpdateCellSizes(DoRepaint: Boolean);
var
  FHOff, FWOff, i: Integer;
begin
  FHOff := 1;
  FWOff := 1;
  FCellXSize := (Width - 2 * FFrameWidth) div FNumX;
  FCellYSize := (Height - 2 * FFrameWidth) div FNumY;
  if (FCellXSize * FNumX + 2 * FFrameWidth) < Width then
     FHOff := (Width - (FCellXSize * FNumX + 2 * FFrameWidth));
  if (FCellYSize * FNumY + 2 * FFrameWidth) < Height then
     FWOff := (Height - (FCellYSize * FNumY + 2 * FFrameWidth));
  if FHOff < 0 then FHOff := 1;
  if FWOff < 0 then FWOff := 1;
  if FHOff < FWOff then
     FFrameWidth := FPrivWidth + FHOff div 2
  else FFrameWidth := FPrivWidth + FWOff div 2;
  UpdateBounds;
  if DoRepaint then Invalidate;
  if FCellGraphic.Count < FNumCell then
    if FCellGraphic.Count > 0 then
        for i := (FNumCell - FCellGraphic.Count) to FNumCell do
        FCellGraphic.Add(nil)
    else
        for i := 0 to FNumCell do
        FCellGraphic.Add(nil);
end;

procedure THSBevelGrid.SetNumX(ANumX : Integer);
begin
   if ANumX > 0 then FNumX := ANumX else FNumX := 1;
   if ANumX < 20 then FNumX := ANumX else FNumX := 20;
   FNumCell := FNumY * FNumX;
   UpdateCellSizes(True);
end;

procedure THSBevelGrid.SetNumY(ANumY : Integer);
begin
   if ANumY > 0 then FNumY := ANumY else FNumY := 1;
   if ANumY < 20 then FNumY := ANumY else FNumY := 20;
   FNumCell := FNumY * FNumX;
   UpdateCellSizes(True);
end;

procedure THSBevelGrid.SetTransparent(ATran : Boolean);
begin
     FTransparent := ATran;
     UpdateCellSizes(True);
end;

procedure THSBevelGrid.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1;
end;

procedure THSBevelGrid.SetBevel(AStyle : TBevelStyle);
begin
  FBevel := AStyle;
  Invalidate;
end;

procedure THSBevelGrid.SetCellBorder(AStyle : TCellBevelStyle);
begin
  FCBorder := AStyle;
  Invalidate;
end;

procedure THSBevelGrid.SetColor(AColor : TColor);
begin
  FColor := AColor;
  Invalidate;
end;

procedure THSBevelGrid.SetCellDiff(AWidth : Integer);
begin
  if AWidth < 0 then AWidth := 0;
  if AWidth > 10 then AWidth := 10;
  FCellDiff := AWidth;
  Invalidate;
end;

procedure THSBevelGrid.SetCellWidth(AWidth : Integer);
begin
  if AWidth < 0 then AWidth := 0;
  FCellXSize := AWidth;
  UpdateCellSizes(True);
end;

procedure THSBevelGrid.SetFrameWidth(AWidth : Integer);
begin
  if AWidth < 0 then AWidth := 0;
  FFrameWidth := AWidth;
  FPrivWidth := AWidth;
  UpdateCellSizes(True);
end;

procedure THSBevelGrid.SetCellHeight(AHeight : Integer);
begin
  if AHeight < 0 then AHeight := 0;
  FCellYSize := AHeight;
  UpdateCellSizes(True);
end;

procedure THSBevelGrid.UpdateBounds;
var
  NewWidth, NewHeight : Integer;
begin
   NewWidth := (FCellXSize * FNumX) + 2 * FFrameWidth;
   NewHeight := (FCellYSize * FNumY) + 2 * FFrameWidth;
   BoundsRect := Bounds(Left, Top, NewWidth, NewHeight);
end;

procedure THSBevelGrid.SetPicture(APicture : TPicture);
begin
  FPicture.Assign(APicture);
  Invalidate;
end;

procedure THSBevelGrid.SetBorder(ABorder : TBorderStyle);
begin
    FBorder := ABorder;
    Invalidate;
end;

procedure THSBevelGrid.SetCellBrushStyle(AStyle : TBrushStyle);
begin
    FCellBrushStyle := AStyle;
    Invalidate;
end;

procedure THSBevelGrid.SetBorderColor(AColor : TColor);
begin
    FBorderColor := AColor;
    Invalidate;
end;

procedure THSBevelGrid.SetBevelColor(AColor : TColor);
begin
    FBevelColor := AColor;
    Invalidate;
end;

procedure THSBevelGrid.SetCellBorderColor(AColor : TColor);
begin
    FCBorderColor := AColor;
    Invalidate;
end;

procedure THSBevelGrid.SetCellIndex(Index: Integer);
var
  Rect : TRect;
  STop, SLeft : integer;
begin
  if Index > FNumCell - 1 then exit;
  FCellIndex := Index;
  STop := (FCellIndex div FNumX) * FCellYSize + FFrameWidth;
  SLeft := (FCellIndex mod FNumX) * FCellXSize + FFrameWidth;
  FRect := Bounds(SLeft, STop, FCellXSize, FCellYSize);
  InflateRect(FRect, - (FCellDiff + 3), - (FCellDiff + 3));
end;

procedure THSBevelGrid.CellGraphic(Index : Integer; AGraphic : TGraphic);
var
  Grp : TGraphic;
begin
    if Index > FCellGraphic.Count - 1 then Exit;
    FCellGraphic.Insert(Index, AGraphic);
    if FCellGraphic.Items[Index + 1] = nil then
       FCellGraphic.Delete(Index + 1) else
    begin
       Grp := FCellGraphic.Items[Index + 1];
       Grp.Free;
       FCellGraphic.Delete(Index + 1);
    end
end;

procedure THSBevelGrid.DrawCellGraphic(Cell: Integer; CellRect : TRect);
 var
   Graphic : TGraphic;
   OffSet1, Offset2 : Integer;
   ICO : TIcon;
begin
    InflateRect(CellRect, - 5, - 5);
    Graphic := FCellGraphic.Items[Cell];
    if Graphic is TBitMap then
      begin
         if (Graphic.Width > (CellRect.Right - CellRect.Left)) or
           (Graphic.Height > (CellRect.Bottom - CellRect.Top)) then
           begin
               Canvas.StretchDraw(CellRect, Graphic);
           end
      else
           begin
               Offset1 := ((CellRect.Right - CellRect.Left) - Graphic.Width) div 2;
               Offset2 := ((CellRect.Bottom - CellRect.Top) - Graphic.Height) div 2;
               Canvas.Draw(CellRect.Left + Offset1, CellRect.Top + Offset2, Graphic);
           end;
      end;
    if Graphic is TIcon then
      begin
         ICO := TIcon.Create;
         ICO.Assign(Graphic);
         Offset1 := ((CellRect.Right - CellRect.Left) - Graphic.Width) div 2;
         Offset2 := ((CellRect.Bottom - CellRect.Top) - Graphic.Height) div 2;
         DrawIcon(Canvas.Handle, CellRect.Left + Offset1, CellRect.Top + Offset2, ICO.Handle);
         ICO.Free;
      end;
    if Graphic is TMetaFile then
      begin
         if (Graphic.Width > (CellRect.Right - CellRect.Left)) or
           (Graphic.Height > (CellRect.Bottom - CellRect.Top)) then
           begin
               Canvas.StretchDraw(CellRect, Graphic);
           end
      else
           begin
               Offset1 := ((CellRect.Right - CellRect.Left) - Graphic.Width) div 2;
               Offset2 := ((CellRect.Bottom - CellRect.Top) - Graphic.Height) div 2;
               Canvas.Draw(CellRect.Left + Offset1, CellRect.Top + Offset2, Graphic);
           end;
      end;
end;

procedure THSBevelGrid.Refresh;
begin
   Invalidate;
end;

procedure THSBevelGrid.DrawPicture(CellRect : TRect);
var
   OffSet1, Offset2 : Integer;
   ICO : TIcon;
   FBMP : TBitmap;
   FICO : TIcon;
   FWMF : TMetafile;
begin
    if FPicture.Graphic is TBitmap then
      begin
         FBMP := FPicture.Bitmap;
         if FStretch = True then Canvas.StretchDraw(CellRect, FBMP)
         else
           begin
               Offset1 := ((CellRect.Right - CellRect.Left) - FBMP.Width) div 2;
               if Offset1 < 0 then Offset1 := 0;
               Offset2 := ((CellRect.Bottom - CellRect.Top) - FBMP.Height) div 2;
               if Offset2 < 0 then Offset2 := 0;
               Canvas.Draw(CellRect.Left + Offset1, CellRect.Top + Offset2, FBMP);
           end;
      end;
    if FPicture.Graphic is TIcon then
      begin
         FICO := FPicture.Icon;
         Offset1 := ((CellRect.Right - CellRect.Left) - FICO.Width) div 2;
         Offset2 := ((CellRect.Bottom - CellRect.Top) - FICO.Height) div 2;
         DrawIcon(Canvas.Handle, CellRect.Left + Offset1, CellRect.Top + Offset2, FICO.Handle);
      end;
    if FPicture.Graphic is TMetafile then
      begin
         FWMF := FPicture.Metafile;
         if FStretch = True then Canvas.StretchDraw(CellRect, FWMF)
         else
           begin
               Offset1 := ((CellRect.Right - CellRect.Left) - FWMF.Width) div 2;
               if Offset1 < 0 then Offset1 := 0;
               Offset2 := ((CellRect.Bottom - CellRect.Top) - FWMF.Height) div 2;
               if Offset2 < 0 then Offset2 := 0;
               Canvas.Draw(CellRect.Left + Offset1, CellRect.Top + Offset2, FWMF);
           end;
      end;
end;

procedure THSBevelGrid.SetStretch(AStretch : Boolean);
begin
   if FStretch = AStretch then Exit;
   FStretch := AStretch;
   if FPicture <> nil then Invalidate;
end;

end.


