unit Hslabel;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls;

const
  HatchBits50: array[0..7] of Word = ($AA, $55,$AA, $55,$AA, $55,$AA, $55);
  HatchBits25: array[0..7] of Word = ($AA, $0,$AA, $0,$AA, $0,$AA, $0);

type
  TShadowPos = (spRightDown, spRightUp, spLeftDown, spLeftUp);
  TFillDirection = (fdUp, fdDown, fdLeft, fdRight);
  TBackground = (bkGradient, bkGlass, bkSolid);
  TTrans = (tr50, tr75);

type
  THSLabel = class(TGraphicControl)
  private
     FBevel, FTransparent : Boolean;
     FCaption : string;
     FStyle : TBevelStyle;
     FOnChange : TNotifyEvent;
     FSColor, FOldColor : TColor;
     FPos : TShadowPos;
     FOffset : Integer;
     FShadow : Boolean;
     FStart, FEnd : TColor;
     FBack : TBackground;
     FDirection : TFillDirection;
     FTrans : TTrans;
    { Private declarations }
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure DrawCaption;
    procedure DrawShadow(S: Boolean; T:Real);
    procedure SetBevel(ABevel : Boolean);
    procedure SetStyle(AStyle : TBevelStyle);
    procedure SetTransparent(ATransparent : Boolean);
    procedure FillWithGradient;
    procedure FillGlass;
    procedure Change;
    procedure SetPos(APos : TShadowPos);
    procedure SetShadow(AShadow : Boolean);
    procedure SetOffset(AOffset : Integer);
    procedure SetSColor(AColor : TColor);
    procedure SetBack(ABack : TBackground);
    procedure SetDirection(ADirection : TFillDirection);
    procedure SetStartColor(AColor : TColor);
    procedure SetEndColor(AColor : TColor);
    procedure SetTrans(ATrans : TTrans);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetCaption(ACaption : string);
    procedure Refresh; virtual;
    { Public declarations }
  published
    { Published declarations }
    property Background : TBackground read Fback write SetBack;
    property FillDirection : TFillDirection read FDirection write SetDirection;
    property StartColor : TColor read FStart write SetStartColor;
    property EndColor : TColor read FEnd write SetEndColor;
    property FillTransparent : TTrans read FTrans write SetTrans;
    property ShadowColor: TColor read FSColor write SetSColor;
    property Shadow : Boolean read FShadow write SetShadow;
    property ShadowSize : Integer read FOffset write SetOffset;
    property ShadowPos : TShadowPos read FPos write SetPos;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property Bevel : Boolean read FBevel write SetBevel;
    property BevelStyle : TBevelStyle read FStyle write SetStyle;
    property Transparent : Boolean read FTransparent write SetTransparent;
    property Font;
    property Caption: string read FCaption write SetCaption;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
end;


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('HS Controls', [THSLabel]);
end;

constructor THSLabel.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  Font.Name := 'Arial';
  Font.Color := clMaroon;
  FShadow := True;
  FPos := spRightDown;
  FOffset := 1;
  FSColor := clWhite;
  FBack := bkGradient;
  FStart := clTeal;
  FEnd := clWhite;
  FDirection := fdDown;
  FTrans := tr75;
  Bevel := False;
  if csDesigning in ComponentState then
  FCaption := 'HSLabel';
  Transparent := False;
end;

destructor THSLabel.Destroy;
begin
  inherited Destroy;
end;

procedure THSLabel.SetShadow(AShadow : Boolean);
begin
   if AShadow = FShadow then exit;
   FShadow := AShadow;
   if FTransparent = False then Invalidate;
end;

procedure THSLabel.SetOffset(AOffset : Integer);
begin
     if AOffset < 1 then AOffset := 1;
     if AOffset > 5 then AOffset := 5;
     FOffset := AOffset;
     Invalidate;
end;

procedure THSLabel.SetPos(APos : TShadowPos);
begin
     FPos := APos;
     Invalidate;
end;

procedure THSLabel.SetSColor(AColor : TColor);
begin
     FSColor := AColor;
     Invalidate;
end;

procedure THSLabel.SetBack(ABack : TBackground);
begin
   FBack := Aback;
   if FTransparent = False then Invalidate;
end;

procedure THSLabel.SetDirection(ADirection : TFillDirection);
begin
    FDirection := ADirection;
    if FBack = bkGradient then Invalidate;
end;

procedure THSLabel.SetStartColor(AColor: TColor);
begin
    FStart := AColor;
    if FTransparent = False then Invalidate;
end;

procedure THSLabel.SetEndColor(AColor: TColor);
begin
    FEnd := AColor;
    if FTransparent = False then Invalidate;
end;

procedure THSLabel.SetTrans(ATrans : TTrans);
begin
    FTrans := ATrans;
    if FTransparent = False then Invalidate;
end;

procedure THSlabel.Paint;
var
   BRect : TRect;
begin
 FOldColor := Font.Color;
 with Canvas do
  begin
    Brush.Color := FStart;
    if FTransparent = False then
       begin
         if FBack = bkGradient then FillWithGradient;
         if FBack = bkGlass then FillGlass;
         if FBack = bkSolid then FillRect(ClipRect);
       end;
    Brush.Color := clBtnFace;
    BRect := Bounds(0, 0, Width, Height);
    if (FBevel = True) and (FTransparent = False) then
      begin
        if FStyle = bsLowered then
           begin
              Frame3D(Canvas, BRect, clBtnShadow, clBtnHighlight, 1);
           end
        else
           begin
              Frame3D(Canvas, BRect, clBtnHighlight, clBtnShadow, 1);
           end;
      end;
   end;
   DrawCaption;
end;

procedure THSLabel.DrawCaption;
   function CheckedCaption(Cap:String): Boolean;
      var i : integer;
     begin
        Result := False;
        for i := 0 to Length(Cap) - 1 do
          begin
             if (Copy(Cap, i + 1, 1) = 'y') or
                (Copy(Cap, i + 1, 1) = 'q') or
                (Copy(Cap, i + 1, 1) = 'j') or
                (Copy(Cap, i + 1, 1) = 'p') then
                begin
                  Result := True;
                  exit;
                end;
          end;
     end;
var
  LogRec: TLOGFONT;
  OldFont, NewFont: HFONT;
  CalculateWidth, FWidth,
  FHeight, FTop : Integer;
  XPos, YPos : Integer;
  Tan, Angle : Real;
  FSmall : Boolean;
  OColor : TColor;
begin
  FSmall := CheckedCaption(FCaption);
  with Canvas do
   begin
    Angle := (3.14159 * 75) / 180;
    Tan := Sin(Angle) / Cos(Angle);
    if FShadow = True then DrawShadow(FSmall, Tan);
    Font := Self.Font;
{    Canvas.Font.Color := FOldColor; }
    CalculateWidth := Width - 4;
    if FShadow then CalculateWidth := CalculateWidth - 2 * FOffset;
    if (Font.Style = [fsItalic]) or (Font.Style = [fsBold, fsItalic]) then
    CalculateWidth := CalculateWidth - Round(Height / Tan);
    Brush.Style := bsClear;
    FWidth := CalculateWidth div Length(Caption);
    if FSmall = True then
        begin
          FHeight := Round(((Height - 2 * FOffset) * 1.2) + Round(Round(Height) div 10) * 2.2);
          FHeight := Round(FHeight * 0.73);
        end
    else FHeight := Round(((Height - 2 * FOffset) * 1.2) + Round((Height div 10) * 2.2));
    GetObject(Font.Handle, SizeOf(LogRec), @LogRec);
    LogRec.lfHeight := FHeight;
    LogRec.lfWidth := FWidth;
    LogRec.lfEscapement := 0;
    LogRec.lfOutPrecision := OUT_TT_PRECIS;
    LogRec.lfClipPrecision := CLIP_TT_ALWAYS;
    LogRec.lfPitchAndFamily := VARIABLE_PITCH;
    NewFont := CreateFontIndirect(LogRec);
    OldFont := SelectObject(Handle, NewFont);
    FWidth := Round(FWidth / (TextWidth(Caption) / CalculateWidth));
    NewFont := SelectObject(Handle, OldFont);
    DeleteObject(NewFont);
    GetObject(Font.Handle, SizeOf(LogRec), @LogRec);
    LogRec.lfHeight := FHeight;
    LogRec.lfWidth := FWidth;
    LogRec.lfEscapement := 0;
    LogRec.lfOutPrecision := OUT_TT_PRECIS;
    LogRec.lfClipPrecision := CLIP_TT_ALWAYS;
    LogRec.lfPitchAndFamily := VARIABLE_PITCH;
    NewFont := CreateFontIndirect(LogRec);
    OldFont := SelectObject(Handle, NewFont);
    if FSmall then
       YPos := - ((TextHeight(FCaption) - Height) + (FOffset + 18)) div 2 + 8
    else YPos := - ((TextHeight(FCaption) - Height) + FOffset) div 2 + 1;
    XPos := FOffset;
    TextOut(XPos, YPos, FCaption);
    NewFont := SelectObject(Handle, OldFont);
    DeleteObject(NewFont);
 end;
end;

procedure THSLabel.DrawShadow(S:Boolean; T:Real);
var
  LogRec: TLOGFONT;
  OldFont, NewFont: HFONT;
  CalculateWidth, FWidth,
  FHeight, FTop : Integer;
  XPos, YPos : Integer;
  OldColor : TColor;
begin
  with Canvas do
   begin
    Font := Self.Font;
    OldColor := Font.Color;
    Font.Color := FSColor;
    CalculateWidth := Width - 4;
    if FShadow then CalculateWidth := CalculateWidth - 2 * FOffset;
    if (Font.Style = [fsItalic]) or (Font.Style = [fsBold, fsItalic]) then
    CalculateWidth := CalculateWidth - Round(Height / T);
    Brush.Style := bsClear;
    FWidth := CalculateWidth div Length(Caption);
    if S = True then
        begin
          FHeight := Round(((Height - 2 * FOffset) * 1.2) + Round(Round(Height) div 10) * 2.2);
          FHeight := Round(FHeight * 0.73);
        end
    else FHeight := Round(((Height - 2 * FOffset) * 1.2) + Round((Height div 10) * 2.2));
    GetObject(Canvas.Font.Handle, SizeOf(LogRec), @LogRec);
    LogRec.lfHeight := FHeight;
    LogRec.lfWidth := FWidth;
    LogRec.lfEscapement := 0;
    LogRec.lfOutPrecision := OUT_TT_PRECIS;
    LogRec.lfClipPrecision := CLIP_TT_ALWAYS;
    LogRec.lfPitchAndFamily := VARIABLE_PITCH;
    NewFont := CreateFontIndirect(LogRec);
    OldFont := SelectObject(Canvas.Handle, NewFont);
    FWidth := Round(FWidth / (TextWidth(Caption) / CalculateWidth));
    NewFont := SelectObject(Canvas.Handle, OldFont);
    DeleteObject(NewFont);
    GetObject(Font.Handle, SizeOf(LogRec), @LogRec);
    LogRec.lfHeight := FHeight;
    LogRec.lfWidth := FWidth;
    LogRec.lfEscapement := 0;
    LogRec.lfOutPrecision := OUT_TT_PRECIS;
    LogRec.lfClipPrecision := CLIP_TT_ALWAYS;
    LogRec.lfPitchAndFamily := VARIABLE_PITCH;
    NewFont := CreateFontIndirect(LogRec);
    OldFont := SelectObject(Canvas.Handle, NewFont);
    if S then
       YPos := - ((TextHeight(FCaption) - Height) + (FOffset + 18)) div 2 + 8
    else YPos := - ((TextHeight(FCaption) - Height) + FOffset) div 2;
    XPos := FOffset;
    Case FPos of
        spRightDown : begin
                        XPos := XPos + FOffset;
                        YPos := YPos + FOffset;
                      end;
          spRightUp : begin
                        XPos := XPos + FOffset;
                        YPos := YPos - FOffset;
                      end;
         spLeftDown : begin
                        XPos := XPos - FOffset;
                        YPos := YPos + FOffset;
                       end;
            spLeftUp : begin
                        XPos := XPos - FOffset;
                        YPos := YPos - FOffset;
                       end;
   end;
    TextOut(XPos, YPos, FCaption);
    NewFont := SelectObject(Canvas.Handle, OldFont);
    DeleteObject(NewFont);
    Font.Color := OldColor;
 end;
end;

procedure THSlabel.SetCaption(ACaption : string);
begin
   FCaption := ACaption;
   Invalidate;
end;

procedure THSLabel.SetBevel(ABevel : Boolean);
begin
   if ABevel = FBevel then exit;
   FBevel := ABevel;
   if FTransparent = False then Invalidate;
end;

procedure THSlabel.SetStyle(AStyle : TBevelStyle);
begin
   if AStyle = FStyle then exit;
   FStyle := AStyle;
   if FBevel = True then Invalidate;
end;

procedure THSLabel.SetTransparent(ATransparent : Boolean);
begin
   if ATransparent = FTransparent then Exit;
   FTransparent := ATransparent;
   Invalidate;
end;

procedure THSLabel.Refresh;
begin
  Invalidate;
end;

procedure THSLabel.Change;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure THSLabel.FillWithGradient;
var
  TargetRect : TRect;
  i         : Integer;
  clrFrom,clrTo : TColor;
  RGBFromR,RGBFromG,RGBFromB : Byte;
  RGBDiffR,RGBDiffG,RGBDiffB : integer;

begin
   clrFrom := FEnd;
   clrTo := FEnd;

   Case FDirection of
   fdUp: clrFrom := FStart;
   fdDown: clrTo := FStart;
   fdLeft: clrFrom := FStart;
   fdRight: clrTo := FStart;
   end;


   RGBFromR := GetRValue (ColorToRGB (ClrFrom));
   RGBFromG := GetGValue (ColorToRGB (ClrFrom));
   RGBFromB := GetBValue (ColorToRGB (ClrFrom));
   RGBDiffR := GetRValue (ColorToRGB (ClrTo)) - RGBFromR;
   RGBDiffG := GetGValue (ColorToRGB (ClrTo)) - RGBFromG;
   RGBDiffB := GetBValue (ColorToRGB (ClrTo)) - RGBFromB;

   Canvas.Pen.Style := psSolid;
   Canvas.Pen.Mode := pmCopy;
   if (FDirection = fdLeft) or (FDirection = fdRight) then
    begin
     for i := 0 to 255 do
     begin
         TargetRect := Rect(MulDiv (i,Width,256),0,
                            MulDiv (i+1,Width,256),Height);
         Canvas.Brush.Color := RGB (RGBFromR + MulDiv(i,RGBDiffR, 255),
                                    RGBFromG + MulDiv(i,RGBDiffG, 255),
                                    RGBFromB + MulDiv(i,RGBDiffB, 255));
         Canvas.FillRect (TargetRect);
     end;
    end
   else
    begin
     for i := 0 to $ff do
     begin
         TargetRect := Rect(0,MulDiv (i,Height,256),
                            Width,MulDiv (i+1,Height,256));
         Canvas.Brush.Color := RGB (RGBFromR + MulDiv(i,RGBDiffR,255),
                                    RGBFromG + MulDiv(i,RGBDiffG,255),
                                    RGBFromB + MulDiv(i,RGBDiffB,255));
         Canvas.FillRect (TargetRect);
     end;
   end;
end;

procedure THSLabel.FillGlass;
  var
    rgn : HRgn;
    i : integer;
    Bitmap: HBitmap;
    SaveBrush: HBrush;
    SaveTextColor, SaveBkColor: TColorRef;
begin
   with Canvas do
     begin
       brush.color := FStart;
       brush.style := bsSolid;
       rgn :=  CreateRectRgn(Left, Top, Left + Width, Top + Height);
       SelectClipRgn(Handle,rgn);
       if FTrans = tr50 then
       Bitmap := CreateBitmap(8, 8, 1, 1, @HatchBits50)
       else
       Bitmap := CreateBitmap(8, 8, 1, 1, @HatchBits25);
       SaveBrush := SelectObject(Handle, CreatePatternBrush(Bitmap));
       SaveTextColor := SetTextColor(Handle, clWhite);
       SaveBkColor := SetBkColor(Handle, FStart);
       PatBlt(Handle, 0, 0, Width, Height, $00A000C9);
       SetBkColor(Handle, SaveBkColor);
       SetTextColor(Handle, SaveTextColor);
       DeleteObject(SelectObject(Handle, SaveBrush));
       DeleteObject(Bitmap);
       SelectClipRgn(Handle,0);
       DeleteObject(rgn);
     end;
end;

end.
